!    This file is part of hybrid.
!    $Id: hybrid_atom.f90,v 1.6 2009-02-10 19:38:22 tburnus Exp $
!
! ***** BEGIN LICENSE BLOCK *****
! Version: MPL 1.1/GPL 2.0/LGPL 2.1
!
! The contents of this file are subject to the Mozilla Public License Version
! 1.1 (the "License"); you may not use this file except in compliance with
! the License. You may obtain a copy of the License at
! http://www.mozilla.org/MPL/
!
! Software distributed under the License is distributed on an "AS IS" basis,
! WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
! for the specific language governing rights and limitations under the
! License.
!
! The Original Code is Hybrid
!
! The Initial Developer of the Original Code is Tobias Burnus <burnus@net-b.de>
! Portions created by the Initial Developer are Copyright (C) 2006-2009
! the Initial Developer. All Rights Reserved.
!
! Contributor(s):
!
! Alternatively, the contents of this file may be used under the terms of
! either the GNU General Public License Version 2 or later (the "GPL"), or
! the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
! in which case the provisions of the GPL or the LGPL are applicable instead
! of those above. If you wish to allow use of your version of this file only
! under the terms of either the GPL or the LGPL, and not to allow others to
! use your version of this file under the terms of the MPL, indicate your
! decision by deleting the provisions above and replace them with the notice
! and other provisions required by the GPL or the LGPL. If you do not delete
! the provisions above, a recipient may use your version of this file under
! the terms of any one of the MPL, the GPL or the LGPL.
!
! ***** END LICENSE BLOCK *****

! Contains:
! - type atom_type
! - function getAtomicNo(elem) - geturns Z = 8 for "O"yxgen
! - subroutine getAtomData(Z, atomData) - returns type atom_type for a Z
!
module atomdata
  use general, only: r8
  implicit none

  type atom_type
     integer :: Z           ! atomic number
     character(3)  :: elem  ! element symbol
     character(20) :: name  ! element name (English)
     integer   :: vshell    ! outer/valence shell: main quantum number (of ion)
     integer   :: vsubshell ! outer/valence shell: subshell q.n.: d = 2 etc.
     ! Example: (22,'Ti', 'Titanium', 3, 2), where 3,2 => "3d"
     real(r8)  :: Es, Ep, Ed ! Tight binding: on-site energy
  end type atom_type

contains

  ! Returns atomic number Z for a given element
  ! Parameter:  elem(3) - elementname
  ! Return:     atomic number
  function getAtomicNo(elem)
     character(len=*), intent(in) :: elem
     integer                      :: getAtomicNo

     select case(elem)
       case('H' ); getAtomicNo =  1
       case('He'); getAtomicNo =  2

       case('Li'); getAtomicNo =  3
       case('Be'); getAtomicNo =  4
       case('B' ); getAtomicNo =  5
       case('C' ); getAtomicNo =  6
       case('N' ); getAtomicNo =  7
       case('O' ); getAtomicNo =  8
       case('F' ); getAtomicNo =  9
       case('Ne'); getAtomicNo = 10

       case('Na'); getAtomicNo = 11
       case('Mg'); getAtomicNo = 12
       case('Al'); getAtomicNo = 13
       case('Si'); getAtomicNo = 14
       case('P' ); getAtomicNo = 15
       case('S' ); getAtomicNo = 16
       case('Cl'); getAtomicNo = 17
       case('Ar'); getAtomicNo = 18

       case('K' ); getAtomicNo = 19
       case('Ca'); getAtomicNo = 20
       case('Sc'); getAtomicNo = 21
       case('Ti'); getAtomicNo = 22
       case('V' ); getAtomicNo = 23
       case('Cr'); getAtomicNo = 24
       case('Mn'); getAtomicNo = 25
       case('Fe'); getAtomicNo = 26
       case('Co'); getAtomicNo = 27
       case('Ni'); getAtomicNo = 28
       case('Cu'); getAtomicNo = 29
       case('Zn'); getAtomicNo = 30
       case('Ga'); getAtomicNo = 31
       case('Ge'); getAtomicNo = 32
       case('As'); getAtomicNo = 33
       case('Se'); getAtomicNo = 34
       case('Br'); getAtomicNo = 35
       case('Kr'); getAtomicNo = 36

       case('Rb'); getAtomicNo = 37
       case('Sr'); getAtomicNo = 38
       case('Y' ); getAtomicNo = 39
       case('Zr'); getAtomicNo = 40
       case('Nb'); getAtomicNo = 41
       case('Mo'); getAtomicNo = 42
       case('Tc'); getAtomicNo = 43
       case('Ru'); getAtomicNo = 44
       case('Rh'); getAtomicNo = 45
       case('Pd'); getAtomicNo = 46
       case('Ag'); getAtomicNo = 47
       case('Cd'); getAtomicNo = 48
       case('In'); getAtomicNo = 49
       case('Sn'); getAtomicNo = 50
       case('Sb'); getAtomicNo = 51
       case('Te'); getAtomicNo = 52
       case('I' ); getAtomicNo = 53
       case('Xe'); getAtomicNo = 54

       case('Cs'); getAtomicNo = 55
       case('Ba'); getAtomicNo = 56
       case('La'); getAtomicNo = 57
       case('Ce'); getAtomicNo = 58
       case('Pr'); getAtomicNo = 59
       case('Nd'); getAtomicNo = 60
       case('Pm'); getAtomicNo = 61
       case('Sm'); getAtomicNo = 62
       case('Eu'); getAtomicNo = 63
       case('Gd'); getAtomicNo = 64
       case('Tb'); getAtomicNo = 65
       case('Dy'); getAtomicNo = 66
       case('Ho'); getAtomicNo = 67
       case('Er'); getAtomicNo = 68
       case('Tm'); getAtomicNo = 69
       case('Yb'); getAtomicNo = 70
       case('Lu'); getAtomicNo = 71
       case('Hf'); getAtomicNo = 72
       case('Ta'); getAtomicNo = 73
       case('W' ); getAtomicNo = 74
       case('Re'); getAtomicNo = 75
       case('Os'); getAtomicNo = 76
       case('Ir'); getAtomicNo = 77
       case('Pt'); getAtomicNo = 78
       case('Au'); getAtomicNo = 79
       case('Hg'); getAtomicNo = 80
       case('Tl'); getAtomicNo = 81
       case('Pb'); getAtomicNo = 82
       case('Bi'); getAtomicNo = 83
       case('Po'); getAtomicNo = 84
       case('At'); getAtomicNo = 85
       case('Rn'); getAtomicNo = 86

       case('Fr'); getAtomicNo = 87
       case('Ra'); getAtomicNo = 88
       case('Ac'); getAtomicNo = 89
       case('Th'); getAtomicNo = 90
       case('Pa'); getAtomicNo = 91
       case('U' ); getAtomicNo = 92
       case('Np'); getAtomicNo = 93
       case('Pu'); getAtomicNo = 94
       case('Am'); getAtomicNo = 95
       case('Cm'); getAtomicNo = 96
       case('Bk'); getAtomicNo = 97
       case('Cf'); getAtomicNo = 98
       case('Es'); getAtomicNo = 99
       case('Fm'); getAtomicNo =100
       case('Md'); getAtomicNo =101
       case('No'); getAtomicNo =102
       case('Lr'); getAtomicNo =103
       case('Rf'); getAtomicNo =104
       case('Db'); getAtomicNo =105
       case('Sg'); getAtomicNo =106
       case('Bh'); getAtomicNo =107
       case('Hs'); getAtomicNo =108
       case('Mt'); getAtomicNo =109
       case('Ds'); getAtomicNo =110
       case('Rg'); getAtomicNo =111
       case('Uub'); getAtomicNo =112
       case('Uuq'); getAtomicNo =114
       case('Uuh'); getAtomicNo =116
       case default
         write(*,'(2a)') 'ERROR: getAtomicNo - Unknown element: ',trim(elem)
         stop 'ERROR: getAtomicNo - unknown element'
     end select
  end function getAtomicNo

  subroutine getAtomData(Z, atomData)
    implicit none
    integer, intent(in)         :: Z
    type(atom_type),intent(out) :: atomData
    select case(Z)
       case(  1); atomData = assignAtomData(  1, 'H',  'Hydrogen', 1,0)
       case(  2); atomData = assignAtomData(  2, 'He', 'Helium', 1,0)
       case(  3); atomData = assignAtomData(  3, 'Li', 'Lithium', 2,0)
       case(  4); atomData = assignAtomData(  4, 'Be', 'Beryllium', 2,0)
       case(  5); atomData = assignAtomData(  5, 'B',  'Boron', 2,1)
       case(  6); atomData = assignAtomData(  6, 'C',  'Carbon', 2,1)
       case(  7); atomData = assignAtomData(  7, 'N',  'Nitrogen', 2,1)
       case(  8); atomData = assignAtomData(  8, 'O',  'Oxygen', 2,1)
       case(  9); atomData = assignAtomData(  9, 'F',  'Fluorine', 2,1)
       case( 10); atomData = assignAtomData( 10, 'Ne', 'Neon', 2,1)

       case( 11); atomData = assignAtomData( 11, 'Na', 'Sodium', 3,0)
       case( 12); atomData = assignAtomData( 12, 'Mg', 'Magnesium', 3,0)
       case( 13); atomData = assignAtomData( 13, 'Al', 'Aluminium', 3,1)
       case( 14); atomData = assignAtomData( 14, 'Si', 'Silicon', 3,1)
       case( 15); atomData = assignAtomData( 15, 'P',  'Phosphorus', 3,1)
       case( 16); atomData = assignAtomData( 16, 'S',  'Sulfur', 3,1)
       case( 17); atomData = assignAtomData( 17, 'Cl', 'Chlorine', 3,1)
       case( 18); atomData = assignAtomData( 18, 'Ar', 'Argon', 3,1)

       case( 19); atomData = assignAtomData( 19, 'K',  'Potassium', 4,1)
       case( 20); atomData = assignAtomData( 20, 'Ca', 'Calcium', 4,1)
       case( 21); atomData = assignAtomData( 21, 'Sc', 'Scandium', 3,2)
       case( 22); atomData = assignAtomData( 22, 'Ti', 'Titanium', 3,2)
       case( 23); atomData = assignAtomData( 23, 'V',  'Vanadium', 3,2)
       case( 24); atomData = assignAtomData( 24, 'Cr', 'Chromium', 3,2)
       case( 25); atomData = assignAtomData( 25, 'Mn', 'Manganese', 3,2)
       case( 26); atomData = assignAtomData( 26, 'Fe', 'Iron', 3,2)
       case( 27); atomData = assignAtomData( 27, 'Co', 'Cobalt', 3,2)
       case( 28); atomData = assignAtomData( 28, 'Ni', 'Nickel', 3,2)
       case( 29); atomData = assignAtomData( 29, 'Cu', 'Copper', 3,2)
       case( 30); atomData = assignAtomData( 30, 'Zn', 'Zinc', 3,2)
       case( 31); atomData = assignAtomData( 31, 'Ga', 'Gallium', 4,1)
       case( 32); atomData = assignAtomData( 32, 'Ge', 'Germanium', 4,1)
       case( 33); atomData = assignAtomData( 33, 'As', 'Arsenic', 4,1)
       case( 34); atomData = assignAtomData( 34, 'Se', 'Selenium', 4,1)
       case( 35); atomData = assignAtomData( 35, 'Br', 'Bromine', 4,1)
       case( 36); atomData = assignAtomData( 36, 'Kr', 'Kryton', 4,1)

       case( 37); atomData = assignAtomData( 37, 'Rb', 'Rubidium', 5,0)
       case( 38); atomData = assignAtomData( 38, 'Sr', 'Strontium', 5,0)
       case( 39); atomData = assignAtomData( 39, 'Y',  'Yttrium', 4,2)
       case( 40); atomData = assignAtomData( 40, 'Zr', 'Zirconium', 4,2)
       case( 41); atomData = assignAtomData( 41, 'Nb', 'Niobium', 4,2)
       case( 42); atomData = assignAtomData( 42, 'Mo', 'Molybdenum', 4,2)
       case( 43); atomData = assignAtomData( 43, 'Tc', 'Technetium', 4,2)
       case( 44); atomData = assignAtomData( 44, 'Ru', 'Ruthenium', 4,2)
       case( 45); atomData = assignAtomData( 45, 'Rh', 'Rhodium', 4,2)
       case( 46); atomData = assignAtomData( 46, 'Pd', 'Palladium', 4,2)
       case( 47); atomData = assignAtomData( 47, 'Ag', 'Silver', 4,2)
       case( 48); atomData = assignAtomData( 48, 'Cd', 'Cadmium', 4,2)
       case( 49); atomData = assignAtomData( 49, 'In', 'Indium', 5,1)
       case( 50); atomData = assignAtomData( 50, 'Sn', 'Tin', 5,1)
       case( 51); atomData = assignAtomData( 51, 'Sb', 'Antimony', 5,1)
       case( 52); atomData = assignAtomData( 52, 'Te', 'Tellurium', 5,1)
       case( 53); atomData = assignAtomData( 53, 'I',  'Iodine', 5,1)
       case( 54); atomData = assignAtomData( 54, 'Xe', 'Xenon', 5,1)

       case( 55); atomData = assignAtomData( 55, 'Cs', 'Cesium', 6,0)
       case( 56); atomData = assignAtomData( 56, 'Ba', 'Barium', 6,0)
       case( 57); atomData = assignAtomData( 57, 'La', 'Lathanum', 5,2)
       case( 58); atomData = assignAtomData( 58, 'Ce', 'Cerium', 4,3)
       case( 59); atomData = assignAtomData( 59, 'Pr', 'Praseodymium', 4,3)
       case( 60); atomData = assignAtomData( 60, 'Nd', 'Neodymium', 4,3)
       case( 61); atomData = assignAtomData( 61, 'Pm', 'Promethium', 4,3)
       case( 62); atomData = assignAtomData( 62, 'Sm', 'Samarium', 4,3)
       case( 63); atomData = assignAtomData( 63, 'Eu', 'Europium', 4,3)
       case( 64); atomData = assignAtomData( 64, 'Gd', 'Gadolinium', 4,3)
       case( 65); atomData = assignAtomData( 65, 'Tb', 'Terbium', 4,3)
       case( 66); atomData = assignAtomData( 66, 'Dy', 'Dysprosium', 4,3)
       case( 67); atomData = assignAtomData( 67, 'Ho', 'Holmium', 4,3)
       case( 68); atomData = assignAtomData( 68, 'Er', 'Erbium', 4,3)
       case( 69); atomData = assignAtomData( 69, 'Tm', 'Thulium', 4,3)
       case( 70); atomData = assignAtomData( 70, 'Yb', 'Ytterbium', 4,3)
       case( 71); atomData = assignAtomData( 71, 'Lu', 'Lutetium', 4,3)
       case( 72); atomData = assignAtomData( 72, 'Hf', 'Hafnium', 5,2)
       case( 73); atomData = assignAtomData( 73, 'Ta', 'Tantalum', 5,2)
       case( 74); atomData = assignAtomData( 74, 'W',  'Tungsten', 5,2)
       case( 75); atomData = assignAtomData( 75, 'Re', 'Rhenium', 5,2)
       case( 76); atomData = assignAtomData( 76, 'Os', 'Osmium', 5,2)
       case( 77); atomData = assignAtomData( 77, 'Ir', 'Iridium', 5,2)
       case( 78); atomData = assignAtomData( 78, 'Pt', 'Platinum', 5,2)
       case( 79); atomData = assignAtomData( 79, 'Au', 'Gold', 5,2)
       case( 80); atomData = assignAtomData( 80, 'Hg', 'Mercury', 5,2)
       case( 81); atomData = assignAtomData( 81, 'Tl', 'Thallium', 6,1)
       case( 82); atomData = assignAtomData( 82, 'Pb', 'Lead', 6,1)
       case( 83); atomData = assignAtomData( 83, 'Bi', 'Bismuth', 6,1)
       case( 84); atomData = assignAtomData( 84, 'Po', 'Polonium', 6,1)
       case( 85); atomData = assignAtomData( 85, 'At', 'Astatine', 6,1)
       case( 86); atomData = assignAtomData( 86, 'Rn', 'Radon', 6,1)

       case( 87); atomData = assignAtomData( 87, 'Fr', 'Francium', 7,0)
       case( 88); atomData = assignAtomData( 88, 'Ra', 'Radium', 7,0)
       case( 89); atomData = assignAtomData( 89, 'Ac', 'Actinium', 6,2)
       case( 90); atomData = assignAtomData( 90, 'Th', 'Thorium', 6,2)
       case( 91); atomData = assignAtomData( 91, 'Pa', 'Protactinium', 5,3)
       case( 92); atomData = assignAtomData( 92, 'U',  'Uranium', 5,3)
       case( 93); atomData = assignAtomData( 93, 'Np', 'Neptunium', 5,3)
       case( 94); atomData = assignAtomData( 94, 'Pu', 'Plutonium', 5,3)
       case( 95); atomData = assignAtomData( 95, 'Am', 'Americium', 5,3)
       case( 96); atomData = assignAtomData( 96, 'Cm', 'Curium', 5,3)
       case( 97); atomData = assignAtomData( 97, 'Bk', 'Berkelium', 5,3)
       case( 98); atomData = assignAtomData( 98, 'Cf', 'Califonium', 5,3)
       case( 99); atomData = assignAtomData( 99, 'Es', 'Einsteinium', 5,3)
       case(100); atomData = assignAtomData(100, 'Fm', 'Fermium', 5,3)
       case(101); atomData = assignAtomData(101, 'Md', 'Mendelevium', 5,3)
       case(102); atomData = assignAtomData(102, 'No', 'Nobelium', 5,3)
       case(103); atomData = assignAtomData(103, 'Lr', 'Lawencium', 5,3)
       case(104); atomData = assignAtomData(104, 'Rf', 'Ratherfordium', 6,2)
       case(105); atomData = assignAtomData(105, 'Db', 'Dubnium', 99,99)  ! configuration unclear
       case(106); atomData = assignAtomData(106, 'Sg', 'Seaborgium', 99,99)! configuration unclear
       case(107); atomData = assignAtomData(107, 'Bh', 'Bohrium', 99,99)! configuration unclear
       case(108); atomData = assignAtomData(108, 'Hs', 'Hassium', 99,99)! configuration unclear
       case(109); atomData = assignAtomData(109, 'Mt', 'Meitnerium', 99,99)! configuration unclear
       case(110); atomData = assignAtomData(110, 'Ds', 'Darmstadtium', 99,99)! configuration unclear
       case(111); atomData = assignAtomData(111, 'Rg', 'Roentgenium', 99,99)! configuration unclear
       case(112); atomData = assignAtomData(112, 'Uub','Ununbium', 99,99)! configuration unclear
       case(114); atomData = assignAtomData(114, 'Uuq','Ununquadium', 99,99)! configuration unclear
       case(116); atomData = assignAtomData(116, 'Uuh','Ununhexium', 99,99)! configuration unclear
       case default
         write(*,'(a,i0)') 'ERROR: getAtomData - Unknown element: Z = ',Z
         stop
    end select
  contains
    function assignAtomData(Z, elem, name, n, l)
      integer, intent(in)  :: Z,n,l
      character(*),intent(in) :: elem, name
      type(atom_type)      :: assignAtomData

      assignAtomData%Z         = Z
      assignAtomData%elem      = elem
      assignAtomData%name      = name
      assignAtomData%vshell    = n
      assignAtomData%vsubshell = l
    end function assignAtomData
  end subroutine getAtomData

end module atomdata
