!    This file is part of hybrid.
!    $Id: hybrid_general.f90,v 1.7 2009-02-10 19:38:22 tburnus Exp $
!
! ***** BEGIN LICENSE BLOCK *****
! Version: MPL 1.1/GPL 2.0/LGPL 2.1
!
! The contents of this file are subject to the Mozilla Public License Version
! 1.1 (the "License"); you may not use this file except in compliance with
! the License. You may obtain a copy of the License at
! http://www.mozilla.org/MPL/
!
! Software distributed under the License is distributed on an "AS IS" basis,
! WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
! for the specific language governing rights and limitations under the
! License.
!
! The Original Code is Hybrid
!
! The Initial Developer of the Original Code is Tobias Burnus <burnus@net-b.de>
! Portions created by the Initial Developer are Copyright (C) 2006-2009
! the Initial Developer. All Rights Reserved.
!
! Contributor(s):
!
! Alternatively, the contents of this file may be used under the terms of
! either the GNU General Public License Version 2 or later (the "GPL"), or
! the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
! in which case the provisions of the GPL or the LGPL are applicable instead
! of those above. If you wish to allow use of your version of this file only
! under the terms of either the GPL or the LGPL, and not to allow others to
! use your version of this file under the terms of the MPL, indicate your
! decision by deleting the provisions above and replace them with the notice
! and other provisions required by the GPL or the LGPL. If you do not delete
! the provisions above, a recipient may use your version of this file under
! the terms of any one of the MPL, the GPL or the LGPL.
!
! ***** END LICENSE BLOCK *****

module general
  implicit none
  integer, parameter :: r8 = kind(0d0) ! double-precision kind constant
  real(r8), parameter :: M_PI  = 3.14159265358979323846_r8
  real(r8), parameter :: M_PI2 = 3.14159265358979323846_r8/2.0_r8
  real(r8), parameter :: degree = M_PI/180.0_r8 ! deg -> rad
contains
  ! Calculate the distance between two vectors
  function distance(x,y)
    implicit none
    real(r8),dimension(3),intent(in) :: x,y
    real(r8)                         :: distance
    distance = sqrt((x(1)-y(1))**2+(x(2)-y(2))**2+(x(3)-y(3))**2)
  end function distance

  function lowercase(string) result(lower)
    character(len=*), intent(in) :: string
    character(len=len(string)) :: lower
    integer :: j
    do j = 1,len(string)
      if(string(j:j) >= "A" .and. string(j:j) <= "Z") then
        lower(j:j) = achar(iachar(string(j:j)) + 32)
      else
        lower(j:j) = string(j:j)
      end if
    end do
  end function lowercase

  function uppercase(string) result(upper)
    character(len=*), intent(in) :: string
    character(len=len(string)) :: upper
    integer :: j
    do j = 1,len(string)
      if(string(j:j) >= "a" .and. string(j:j) <= "z") then
           upper(j:j) = achar(iachar(string(j:j)) - 32)
      else
           upper(j:j) = string(j:j)
      end if
    end do
  end function uppercase

! Variant, ICE with older gfortran
! character (len=20) function Up (string)
! character(len=*) string
! uppercase =                                                                  &
!    transfer(merge(achar(iachar(transfer(string,"x",len(string)))-     &
!    (ichar('a')-ichar('A')) ),                                         &
!    transfer(string,"x",len(string)) ,                                 &
!    transfer(string,"x",len(string)) >= "a" .and.                      &
!    transfer(string,"x",len(string)) <= "z"), repeat("x", len(string)))
! return
! end function Up

end module general
