!    This file is part of hybrid.
!    $Id: hybrid_hybrid.f90,v 1.14 2009-02-10 19:38:22 tburnus Exp $
!
! ***** BEGIN LICENSE BLOCK *****
! Version: MPL 1.1/GPL 2.0/LGPL 2.1
!
! The contents of this file are subject to the Mozilla Public License Version
! 1.1 (the "License"); you may not use this file except in compliance with
! the License. You may obtain a copy of the License at
! http://www.mozilla.org/MPL/
!
! Software distributed under the License is distributed on an "AS IS" basis,
! WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
! for the specific language governing rights and limitations under the
! License.
!
! The Original Code is Hybrid
!
! The Initial Developer of the Original Code is Tobias Burnus <burnus@net-b.de>
! Portions created by the Initial Developer are Copyright (C) 2006-2009
! the Initial Developer. All Rights Reserved.
!
! Contributor(s):
!
! Alternatively, the contents of this file may be used under the terms of
! either the GNU General Public License Version 2 or later (the "GPL"), or
! the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
! in which case the provisions of the GPL or the LGPL are applicable instead
! of those above. If you wish to allow use of your version of this file only
! under the terms of either the GPL or the LGPL, and not to allow others to
! use your version of this file under the terms of the MPL, indicate your
! decision by deleting the provisions above and replace them with the notice
! and other provisions required by the GPL or the LGPL. If you do not delete
! the provisions above, a recipient may use your version of this file under
! the terms of any one of the MPL, the GPL or the LGPL.
!
! ***** END LICENSE BLOCK *****

module hybridization
! contains:
! subroutine solvePDHyb - solve hybridization in spherical harmonics
! subroutine showHybrid(natoms,tatoms,catoms,Vhyb) - show hybridization between
!     two atoms for all their orbitals
! function distance(x,y) - calculate distance
! subroutine printHyb(str,orb1,orb2,hybrid) - print the set type(hyb_type)

contains

subroutine solvePDHyb(natoms,catoms,tatoms,Vhyb,options)
  use general, only: r8,distance
  use atomdata, only: atom_type
  use potential, only: hybPD,expandSphH
  use slaterKoster, only: HarrisonRd, Vpps, Vppp, &
                          Vpds, Vpdp, &
                          labelDt, Epp, Epd,&
                          hyb_type
  use hybrid_read, only: opt_type
  implicit none
  integer, intent(in)       :: natoms
  type(atom_type), pointer  :: tatoms(:)   ! Atom data (intent(in))
  real(r8),        pointer  :: catoms(:,:) ! Atom coordinate (intent(in))
  type(hyb_type),  pointer  :: Vhyb(:,:)   ! user-defined hybridization (intent(in))
  type(opt_type), intent(in):: options

  integer  :: i, j, np, l, pt1, pt2, dt1, m, n
  real(r8) :: dist, rd, fVpps, fVppp, fVpds, fVpdp
  real(r8), allocatable :: Hin(:,:),Hout(:,:) ! Matrix
  type(hyb_type)  :: VhybTmp
  complex(r8)     :: B(0:4,-4:4)
  logical         :: l_exists

  !
  !  For p--d hybridization, create matrix
  !
  j = 0
  np = 0
  do i = 1,natoms
    if(tatoms(i)%vsubshell == 2) j = j + 1
    if(tatoms(i)%vsubshell == 1) np = np + 1
  end do
  if(j /= 0 .and. np /= 0) then
    print *,''
    print *,'Found d-p hybridization'
    print *,''
    allocate(Hin(np*3,5+np*3),Hout(5,10)) ! allocate (#p-orbitals)x(5) matrix
    Hin = 0.0_r8
    do i = 1,natoms
      if(tatoms(i)%vsubshell == 2) then
          print '(4a,i0,a,i0,a)', trim(tatoms(i)%name),' (',trim(tatoms(i)%elem),',',i,') - ',tatoms(i)%vshell,'d'
          rd = HarrisonRd(tatoms(i)%Z)
!          print *, 'rd(',trim(tatoms(i)%elem),') = ',rd
!          print '(a15,5a14)',(trim(labelDt(j)) ,j = 1, 5)
          l = 0
          do j = 1, natoms
            if(tatoms(j)%vsubshell == 1) then
               l = l + 1
               dist = distance(catoms(:,i),catoms(:,j))
!               print *, 'Vpd(sigma) = ',Vpds(dist,rd),' Vpd(pi) = ',Vpdp(dist,rd)
!               call printHyb("User defined: ",tatoms(i)%vsubshell,tatoms(j)%vsubshell,Vhyb(i,j))
               VhybTmp%Vpds = Vpds(dist,rd)
               VhybTmp%Vpdp = Vpdp(dist,rd)

               if(Vhyb(i,j)%Vpds /= huge(Vhyb(i,j)%Vpds).and.Vhyb(i,j)%fVpds /= huge(Vhyb(i,j)%fVpds)) then
                  write(*,*) "WARNING: Specified both pds and fpds! Using pds"
               end if
               if(Vhyb(i,j)%fVpds /= huge(Vhyb(i,j)%fVpds)) VhybTmp%Vpds = Vhyb(i,j)%fVpds * VhybTmp%Vpds
               if(Vhyb(i,j)%Vpds /= huge(Vhyb(i,j)%Vpds)) VhybTmp%Vpds = Vhyb(i,j)%Vpds

               if(Vhyb(i,j)%Vpdp /= huge(Vhyb(i,j)%Vpdp).and.Vhyb(i,j)%fVpdp /= huge(Vhyb(i,j)%fVpdp)) then
                  write(*,*) "WARNING: Specified both pdp and fpdp! Using pdp"
               end if
               if(Vhyb(i,j)%fVpdp /= huge(Vhyb(i,j)%fVpdp)) VhybTmp%Vpdp = Vhyb(i,j)%fVpdp * VhybTmp%Vpdp
               if(Vhyb(i,j)%Vpdp /= huge(Vhyb(i,j)%Vpdp)) VhybTmp%Vpdp = Vhyb(i,j)%Vpdp

               do pt1 = 1,3
                 do dt1 = 1,5
                   call Epd((catoms(1,i)-catoms(1,j))/dist, &
                            (catoms(2,i)-catoms(2,j))/dist, &
                            (catoms(3,i)-catoms(3,j))/dist, &
                             pt1,dt1,fVpds,fVpdp)
                   Hin(3*(l-1)+pt1,dt1) = fVpds*VhybTmp%Vpds + fVpdp*VhybTmp%Vpdp
                 end do
!                 print '(i2,3a,i2,2a,5f13.8)', l,' (',trim(tatoms(j)%elem),',',j,'), ',trim(labelpt(pt1)), Hin(3*(l-1)+pt1,1:5)
               end do
               ! p-p hybridization
               n = l
               do m = j+1, natoms
                 if(tatoms(m)%vsubshell == 1) then
                   n = n + 1
                   do pt1 = 1,3
                     do pt2 = 1,3
                       dist = distance(catoms(:,j),catoms(:,m))
                       VhybTmp%Vpps = Vpps(dist)
                       VhybTmp%Vppp = Vppp(dist)
                       if(Vhyb(m,j)%Vpps /= huge(Vhyb(m,j)%Vpps).and.Vhyb(m,j)%fVpps /= huge(Vhyb(i,j)%fVpps)) then
                         write(*,*) "WARNING: Specified both pps and fpps! Using pps"
                       end if
                       if(Vhyb(m,j)%fVpps /= huge(Vhyb(m,j)%fVpps)) VhybTmp%Vpps = Vhyb(m,j)%fVpps * VhybTmp%Vpps
                       if(Vhyb(m,j)%Vpps /= huge(Vhyb(m,j)%Vpps)) VhybTmp%Vpps = Vhyb(m,j)%Vpps

                       if(Vhyb(m,j)%Vppp /= huge(Vhyb(m,j)%Vppp).and.Vhyb(m,j)%fVppp /= huge(Vhyb(i,j)%fVppp)) then
                         write(*,*) "WARNING: Specified both ppp and fppp! Using ppp"
                       end if
                       if(Vhyb(m,j)%fVppp /= huge(Vhyb(m,j)%fVppp)) VhybTmp%Vppp = Vhyb(m,j)%fVppp * VhybTmp%Vppp
                       if(Vhyb(m,j)%Vppp /= huge(Vhyb(m,j)%Vppp)) VhybTmp%Vppp = Vhyb(m,j)%Vppp

                       call Epp((/(catoms(1,j)-catoms(1,m))/dist, &
                                  (catoms(2,j)-catoms(2,m))/dist, &
                                  (catoms(3,j)-catoms(3,m))/dist/), &
                                 pt1,pt2,fVpps,fVppp)
                       Hin(3*(l-1)+pt1,5+3*(n-1)+pt2) = fVpps*VhybTmp%Vpps + fVppp*VhybTmp%Vppp
                       Hin(3*(n-1)+pt2,5+3*(l-1)+pt1) = fVpps*VhybTmp%Vpps + fVppp*VhybTmp%Vppp
                     end do
                   end do
                 end if
               end do
            end if
          end do
          call hybPD(np*3,Hin,Hout,options%maxiter)
          print '(/a/)','Solved - result'
          l_exists = .false.
          do j = 1,5
            if(Hout(j,j) < 0.0_r8) l_exists = .true.
          end do
          if(l_exists) then
             print *, 'WARNING: Diagnonal of submatrix not positive.'
          end if
          print *,'----------------- p-d Hybridization ------------'
          print '(a5,5a14)','', ('d_'//trim(labelDt(l)) ,l = 1, 5)
          do j = 1,5
            print '(a9,5f14.8)','Ld_'//trim(labelDt(j)), Hout(j,1:5)
          end do

          print *, '-------- Expanding into spherical harmonics -- coefficients ---------'
          call expandSphH(Hout(1:5,1:5), B,l_exists)
          if(l_exists .eqv. .true.) then ! complex coeff.
            do j = 0, 4
              do l = -j,j
                if(abs(B(j,l)) > tiny(Hout(1,1))*1e3_r8) &
                  print '(a,i1,a,i2,a,f15.8,"  ",f15.8,")")', 'B(',j,',',l,') = (',B(j,l)
              end do
            end do
          else ! real coeff.
            do j = 0, 4
              do l = -j,j
                if(abs(B(j,l)) > tiny(Hout(1,1))*1e3_r8) &
                  print '(a,i1,a,i2,a,f15.8)', 'B(',j,',',l,') = ',real(B(j,l),r8)
              end do
            end do
          end if

          print '(/a)','----------------- Ligand Hybridization ------------'
          print '(a5,5a14)','', ('Ld_'//trim(labelDt(l)) ,l = 1, 5)
          do j = 1,5
            print '(a9,5f14.8)','Ld_'//trim(labelDt(j)), Hout(j,6:10)
          end do
          print *, '-------- Expanding into spherical harmonics -- coefficients ---------'
          call expandSphH(Hout(1:5,6:10), B,l_exists)
          if(l_exists .eqv. .true.) then ! complex coeff.
            do j = 0, 4
              do l = -j,j
                if(abs(B(j,l)) > tiny(Hout(1,1))*1e3_r8) &
                  print '(a,i1,a,i2,a,f15.8,"  ",f15.8,")")', 'B(',j,',',l,') = (',B(j,l)
              end do
            end do
          else ! real coeff.
            do j = 0, 4
              do l = -j,j
                if(abs(B(j,l)) > tiny(Hout(1,1))*1e3_r8) &
                  print '(a,i1,a,i2,a,f15.8)', 'B(',j,',',l,') = ',real(B(j,l),r8)
              end do
            end do
          end if
      end if
    end do
    deallocate(Hin,Hout)
  end if
end subroutine solvePDHyb

subroutine showHybrid(natoms,tatoms,catoms,Vhyb)
  use general, only: r8,distance
  use atomdata, only: ATOM_TYPE
  use slaterKoster, only: HarrisonRd, Vsss, Vsps, Vpps, Vppp, &
                          Vsds, Vpds, Vpdp, Vdds, Vddp, Vddd, &
                          labelPt, labelDt, Esp, Epp, Esd, Epd,Edd,&
                          hyb_type
  implicit none
  integer, intent(in)        :: natoms
  type(atom_type),intent(in) :: tatoms(natoms)   ! Atom data
  real(r8),       intent(in) :: catoms(3,natoms) ! Atom coordinate
  type(hyb_type), intent(in) :: Vhyb(natoms,natoms)   ! user-defined hybridization

  integer :: i, j, pt1, pt2, dt1, dt2
  real(r8)          :: fVsps, fVpps, fVppp, fVsds, fVpds, fVpdp, fVdds, fVddp, fVddd
  real(r8) :: dist,rd,rd2

  do i = 1,natoms
     do j = i+1,natoms
        dist = distance(catoms(:,i),catoms(:,j))
        print *, ''
        print '(3a,i0,3a,i0,a,f0.3,a)', '(',trim(tatoms(i)%elem),',',i,')--(',trim(tatoms(j)%elem),',',j,&
                  ') hybridization (distance=',dist,')'
        print *, '(l=',(catoms(1,i)-catoms(1,j))/dist, &
                ', m=',(catoms(2,i)-catoms(2,j))/dist, &
                ', n=',(catoms(3,i)-catoms(3,j))/dist,')'
        if(tatoms(i)%vsubshell == 0 .and. tatoms(j)%vsubshell == 0) then
           print *, 'Vss(sigma) = ',Vsss(dist)
           call printHyb("User defined: ",tatoms(i)%vsubshell, &
                         tatoms(j)%vsubshell,Vhyb(i,j),Vsss=Vsss(dist))
           print *, 'E_(s,s) = Vsss'
        elseif(tatoms(i)%vsubshell == 0 .and. tatoms(j)%vsubshell == 1 &
           .or.tatoms(i)%vsubshell == 1 .and. tatoms(j)%vsubshell == 0) then
           print *, 'Vsp(sigma) = ',Vsps(dist)
           call printHyb("User defined: ",tatoms(i)%vsubshell,tatoms(j)%vsubshell, &
                         Vhyb(i,j),Vsps=Vsps(dist))
           do pt1 = 1,3
             call Esp((/catoms(1,i)-catoms(1,j), &
                        catoms(2,i)-catoms(2,j), &
                        catoms(3,i)-catoms(3,j)/)/dist, &
                       pt1,fVsps)
             if(abs(fVsps) < 1e-10_r8) then
               print *, 'E_(s,',labelPt(pt1),') = 0'
             else
               print *, 'E_(s,',labelPt(pt1),') = ',fVsps
             end if
           end do
        elseif(tatoms(i)%vsubshell == 1 .and. tatoms(j)%vsubshell == 1) then
           print *, 'Vpp(sigma) = ',Vpps(dist),' Vpp(pi) = ', Vppp(dist)
           call printHyb("User defined: ",tatoms(i)%vsubshell,tatoms(j)%vsubshell,Vhyb(i,j), &
                         Vpps=Vpps(dist), Vppp=Vppp(dist))
           do pt1 = 1,3
             do pt2 = 1,3
                call Epp((/catoms(1,i)-catoms(1,j), &
                           catoms(2,i)-catoms(2,j), &
                           catoms(3,i)-catoms(3,j)/)/dist, &
                         pt1,pt2,fVpps,fVppp)
                if(abs(fVpps)+abs(fVppp) < 1e-10_r8) then
                   print *, 'E_(',labelPt(pt1),',',labelPt(pt2),') = 0'
                else
                   print *, 'E_(',labelPt(pt1),',',labelPt(pt2),') = ',fVpps,' Vpps + ',fVppp,' Vppp'
                end if
             end do
           end do
        elseif(tatoms(i)%vsubshell == 0 .and. tatoms(j)%vsubshell == 2 &
           .or.tatoms(i)%vsubshell == 2 .and. tatoms(j)%vsubshell == 0) then
           if(tatoms(i)%vsubshell == 2) then
             rd = HarrisonRd(tatoms(i)%Z)
             print *, 'rd(',trim(tatoms(i)%elem),') = ',rd
           else
             rd = HarrisonRd(tatoms(j)%Z)
             print *, 'rd(',trim(tatoms(j)%elem),') = ',rd
           end if
           print *, 'Vsd(sigma) = ',Vsds(dist,rd)
           call printHyb("User defined: ",tatoms(i)%vsubshell, &
                         tatoms(j)%vsubshell,Vhyb(i,j), Vsds=Vsds(dist,rd))
           do dt1 = 1,5
             call Esd((catoms(1,i)-catoms(1,j))/dist, &
                      (catoms(2,i)-catoms(2,j))/dist, &
                      (catoms(3,i)-catoms(3,j))/dist, &
                      dt1,fVsds)
             if(abs(fVsds) < 1e-10_r8) then
                print *, 'E_(s,',trim(labelDt(dt1)),') = 0'
             else
                print *, 'E_(s,',trim(labelDt(dt1)),') = ',fVsds
             end if
           end do
        elseif(tatoms(i)%vsubshell == 1 .and. tatoms(j)%vsubshell == 2 &
           .or.tatoms(i)%vsubshell == 2 .and. tatoms(j)%vsubshell == 1) then
           if(tatoms(i)%vsubshell == 2) then
             rd = HarrisonRd(tatoms(i)%Z)
             print *, 'rd(',trim(tatoms(i)%elem),') = ',rd
           else
             rd = HarrisonRd(tatoms(j)%Z)
             print *, 'rd(',trim(tatoms(j)%elem),') = ',rd
           end if
           print *, 'Vpd(sigma) = ',Vpds(dist,rd),' Vpd(pi) = ',Vpdp(dist,rd)
           call printHyb("User defined: ",tatoms(i)%vsubshell,&
                         tatoms(j)%vsubshell,Vhyb(i,j), &
                         Vpds=Vpds(dist,rd), Vpdp=Vpdp(dist,rd))
           do pt1 = 1,3
             do dt2 = 1,5
                call Epd((catoms(1,i)-catoms(1,j))/dist, &
                         (catoms(2,i)-catoms(2,j))/dist, &
                         (catoms(3,i)-catoms(3,j))/dist, &
                         pt1,dt2,fVpds,fVpdp)
                if(abs(fVpds)+abs(fVpdp) < 1e-10_r8) then
                   print *, 'E_(',labelPt(pt1),',',trim(labelDt(dt2)),') = 0'
                else
                   print *, 'E_(',labelPt(pt1),',',trim(labelDt(dt2)),') = ',fVpds,' Vpds + ',fVpdp,' Vpdp'
                end if
             end do
           end do
        elseif(tatoms(i)%vsubshell == 2 .and. tatoms(j)%vsubshell == 2) then
           rd = HarrisonRd(tatoms(i)%Z)
           print *, 'rd(',trim(tatoms(i)%elem),') = ',rd
           if(tatoms(i)%Z /= tatoms(j)%Z) then
             rd2 = HarrisonRd(tatoms(j)%Z)
             print *, 'rd(',trim(tatoms(j)%elem),') = ',rd2
             rd = 0.5_r8*(rd+rd2)
             print *, 'Using average rd: ',rd
           end if
           print *, 'Vdd(sigma) = ',Vdds(dist,rd),' Vdd(pi) = ',Vddp(dist,rd),' Vdd(delta) = ',Vddd(dist,rd)
           call printHyb("User defined: ",tatoms(i)%vsubshell, &
                         tatoms(j)%vsubshell,Vhyb(i,j), &
                         Vdds=Vdds(dist,rd), Vddp=Vddp(dist,rd), Vddd = Vddd(dist,rd))
           if(tatoms(i)%Z /= tatoms(j)%Z) then
             rd = HarrisonRd(tatoms(j)%Z)
             print *, 'rd(',trim(tatoms(j)%elem),') = ',rd
             print *, 'Vdd(sigma) = ',Vdds(dist,rd),' Vdd(pi) = ',Vddp(dist,rd),' Vdd(delta) = ',Vddd(dist,rd)
           end if
           do dt1 = 1,5
             do dt2 = 1,5
                call Edd((catoms(1,i)-catoms(1,j))/dist, &
                         (catoms(2,i)-catoms(2,j))/dist, &
                         (catoms(3,i)-catoms(3,j))/dist, &
                         dt1,dt2,fVdds,fVddp,fVddd)
                if(abs(fVdds)+abs(fVddp)+abs(fVddd) < 1e-10_r8) then
                  print *, 'E_(',trim(labelDt(dt1)),',',trim(labelDt(dt2)),') = 0'
                else
                  print *, 'E_(',trim(labelDt(dt1)),',',trim(labelDt(dt2)),') = ',fVdds,' Vdds + ',fVddp,' Vddp + ',fVddd,' Vddd'
                end if
             end do
           end do
        end if
     end do
  end do
  end subroutine showHybrid

  subroutine printHyb(str,orb1,orb2,hybrid, Vsss, Vsps, Vpps, Vppp, &
                      Vsds, Vpds, Vpdp, Vdds, Vddp, Vddd)
   use general, only: r8
   use slaterKoster, only: hyb_type
   implicit none
   character(*), intent(in)   :: str     ! Prefix string
   integer, intent(in)        :: orb1,orb2 ! orbit type: 0 = s, 1 = p, etc.
   type(hyb_type), intent(in) :: hybrid
   real(r8), intent(in), optional :: Vsss, Vsps, Vpps, Vppp, Vsds, Vpds, &
                                     Vpdp, Vdds, Vddp, Vddd
   real(r8)  :: unset = huge(hybrid%Vsss)
   integer :: atom1, atom2
   if(orb1 > orb2 .or. orb1 == -1) then
     atom1 = orb1
     atom2 = orb2
   else
     atom2 = orb1
     atom1 = orb2
   end if
   select case(atom1)
     case(0)
       if(atom2 == 0 .and. hybrid%Vsss /= unset) write(*,*) trim(str),' Vsss = ', hybrid%Vsss
       if(atom2 == 0 .and. hybrid%fVsss /= unset) write(*,*) trim(str),' Vsss factor = ', hybrid%fVsss
       if(atom2 == 0 .and. hybrid%fVsss /= unset .and. present(Vsss)) &
          write(*,*) trim(str),' User Vsss = ', hybrid%fVsss*Vsss
     case(1)
       if(atom2 == 0 .and. hybrid%Vsps /= unset) write(*,*) trim(str),' Vsps = ', hybrid%Vsps
       if(atom2 == 1 .and. hybrid%Vpps /= unset) write(*,*) trim(str),' Vpps = ', hybrid%Vpps
       if(atom2 == 1 .and. hybrid%Vppp /= unset) write(*,*) trim(str),' Vppp = ', hybrid%Vppp

       if(atom2 == 0 .and. hybrid%fVsps /= unset) write(*,*) trim(str),' Vsps factor = ', hybrid%fVsps
       if(atom2 == 1 .and. hybrid%fVpps /= unset) write(*,*) trim(str),' Vpps factor = ', hybrid%fVpps
       if(atom2 == 1 .and. hybrid%fVppp /= unset) write(*,*) trim(str),' Vppp factor = ', hybrid%fVppp

       if(atom2 == 0 .and. hybrid%fVsps /= unset .and. present(Vsps)) &
         write(*,*) trim(str),' User Vsps = ', hybrid%fVsps*Vsps
       if(atom2 == 1 .and. hybrid%fVpps /= unset .and. present(Vpps)) &
         write(*,*) trim(str),' User Vpps = ', hybrid%fVpps*Vpps
       if(atom2 == 1 .and. hybrid%fVppp /= unset .and. present(Vppp)) &
          write(*,*) trim(str),' User Vppp = ', hybrid%fVppp*Vppp
     case(2)
       if(atom2 == 0 .and. hybrid%Vsds /= unset) write(*,*) trim(str),' Vsds = ', hybrid%Vsds
       if(atom2 == 1 .and. hybrid%Vpds /= unset) write(*,*) trim(str),' Vpds = ', hybrid%Vpds
       if(atom2 == 1 .and. hybrid%Vpdp /= unset) write(*,*) trim(str),' Vpdp = ', hybrid%Vpdp
       if(atom2 == 2 .and. hybrid%Vdds /= unset) write(*,*) trim(str),' Vdds = ', hybrid%Vdds
       if(atom2 == 2 .and. hybrid%Vddp /= unset) write(*,*) trim(str),' Vddp = ', hybrid%Vddp
       if(atom2 == 2 .and. hybrid%Vddd /= unset) write(*,*) trim(str),' Vddd = ', hybrid%Vddd

       if(atom2 == 0 .and. hybrid%fVsds /= unset) write(*,*) trim(str),' Vsds factor = ', hybrid%fVsds
       if(atom2 == 1 .and. hybrid%fVpds /= unset) write(*,*) trim(str),' Vpds factor = ', hybrid%fVpds
       if(atom2 == 1 .and. hybrid%fVpdp /= unset) write(*,*) trim(str),' Vpdp factor = ', hybrid%fVpdp
       if(atom2 == 2 .and. hybrid%fVdds /= unset) write(*,*) trim(str),' Vdds factor = ', hybrid%fVdds
       if(atom2 == 2 .and. hybrid%fVddp /= unset) write(*,*) trim(str),' Vddp factor = ', hybrid%fVddp
       if(atom2 == 2 .and. hybrid%fVddd /= unset) write(*,*) trim(str),' Vddd factor = ', hybrid%fVddd

       if(atom2 == 0 .and. hybrid%fVsds /= unset .and. present(Vsds)) &
          write(*,*) trim(str),' User Vsds = ', hybrid%fVsds*Vsds
       if(atom2 == 1 .and. hybrid%fVpds /= unset .and. present(Vpds)) &
          write(*,*) trim(str),' User Vpds = ', hybrid%fVpds*Vpds
       if(atom2 == 1 .and. hybrid%fVpdp /= unset .and. present(Vpdp)) &
          write(*,*) trim(str),' User Vpdp = ', hybrid%fVpdp*Vpdp
       if(atom2 == 2 .and. hybrid%fVdds /= unset .and. present(Vdds)) &
          write(*,*) trim(str),' User Vdds = ', hybrid%fVdds*Vdds
       if(atom2 == 2 .and. hybrid%fVddp /= unset .and. present(Vddp)) &
          write(*,*) trim(str),' User Vddp = ', hybrid%fVddp*Vddp
       if(atom2 == 2 .and. hybrid%fVddd /= unset .and. present(Vddd)) &
          write(*,*) trim(str),' User Vddd = ', hybrid%fVddd*Vddd
     case(-1) ! show all
       if(hybrid%Vsss /= unset) write(*,*) trim(str),' Vsss = ', hybrid%Vsss
       if(hybrid%Vsps /= unset) write(*,*) trim(str),' Vsps = ', hybrid%Vsps
       if(hybrid%Vpps /= unset) write(*,*) trim(str),' Vpps = ', hybrid%Vpps
       if(hybrid%Vppp /= unset) write(*,*) trim(str),' Vppp = ', hybrid%Vppp
       if(hybrid%Vsds /= unset) write(*,*) trim(str),' Vsds = ', hybrid%Vsds
       if(hybrid%Vpds /= unset) write(*,*) trim(str),' Vpds = ', hybrid%Vpds
       if(hybrid%Vpdp /= unset) write(*,*) trim(str),' Vpdp = ', hybrid%Vpdp
       if(hybrid%Vdds /= unset) write(*,*) trim(str),' Vdds = ', hybrid%Vdds
       if(hybrid%Vddp /= unset) write(*,*) trim(str),' Vddp = ', hybrid%Vddp
       if(hybrid%Vddd /= unset) write(*,*) trim(str),' Vddd = ', hybrid%Vddd

       if(hybrid%fVsss /= unset) write(*,*) trim(str),' Vsss factor = ', hybrid%fVsss
       if(hybrid%fVsps /= unset) write(*,*) trim(str),' Vsps factor = ', hybrid%fVsps
       if(hybrid%fVpps /= unset) write(*,*) trim(str),' Vpps factor = ', hybrid%fVpps
       if(hybrid%fVppp /= unset) write(*,*) trim(str),' Vppp factor = ', hybrid%fVppp
       if(hybrid%fVsds /= unset) write(*,*) trim(str),' Vsds factor = ', hybrid%fVsds
       if(hybrid%fVpds /= unset) write(*,*) trim(str),' Vpds factor = ', hybrid%fVpds
       if(hybrid%fVpdp /= unset) write(*,*) trim(str),' Vpdp factor = ', hybrid%fVpdp
       if(hybrid%fVdds /= unset) write(*,*) trim(str),' Vdds factor = ', hybrid%fVdds
       if(hybrid%fVddp /= unset) write(*,*) trim(str),' Vddp factor = ', hybrid%fVddp
       if(hybrid%fVddd /= unset) write(*,*) trim(str),' Vddd factor = ', hybrid%fVddd

       if(hybrid%fVsss /= unset .and. present(Vsss)) &
         write(*,*) trim(str),' User Vsss = ', hybrid%fVsss*Vsss
       if(hybrid%fVsps /= unset .and. present(Vsps)) &
         write(*,*) trim(str),' User Vsps = ', hybrid%fVsps*Vsps
       if(hybrid%fVpps /= unset .and. present(Vpps)) &
         write(*,*) trim(str),' User Vpps = ', hybrid%fVpps*Vpps
       if(hybrid%fVppp /= unset .and. present(Vppp)) &
         write(*,*) trim(str),' User Vppp = ', hybrid%fVppp*Vppp
       if(hybrid%fVsds /= unset .and. present(Vsds)) &
         write(*,*) trim(str),' User Vsds = ', hybrid%fVsds*Vsds
       if(hybrid%fVpds /= unset .and. present(Vpds)) &
         write(*,*) trim(str),' User Vpds = ', hybrid%fVpds*Vpds
       if(hybrid%fVpdp /= unset .and. present(Vpdp)) &
         write(*,*) trim(str),' User Vpdp = ', hybrid%fVpdp*Vpdp
       if(hybrid%fVdds /= unset .and. present(Vdds)) &
         write(*,*) trim(str),' User Vdds = ', hybrid%fVdds*Vdds
       if(hybrid%fVddp /= unset .and. present(Vddp)) &
         write(*,*) trim(str),' User Vddp = ', hybrid%fVddp*Vddp
       if(hybrid%fVddd /= unset .and. present(Vddd)) &
         write(*,*) trim(str),' User Vddd = ', hybrid%fVddd*Vddd
     case default
       write(*,*) 'ERROR: printHyb - unknown atom type: ',atom1
       stop
   end select
  end subroutine printHyb

end module hybridization
