
!    This file is part of hybrid.
!    $Id: hybrid_read.f90,v 1.14 2009-02-10 19:38:22 tburnus Exp $
!
! ***** BEGIN LICENSE BLOCK *****
! Version: MPL 1.1/GPL 2.0/LGPL 2.1
!
! The contents of this file are subject to the Mozilla Public License Version
! 1.1 (the "License"); you may not use this file except in compliance with
! the License. You may obtain a copy of the License at
! http://www.mozilla.org/MPL/
!
! Software distributed under the License is distributed on an "AS IS" basis,
! WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
! for the specific language governing rights and limitations under the
! License.
!
! The Original Code is Hybrid
!
! The Initial Developer of the Original Code is Tobias Burnus <burnus@net-b.de>
! Portions created by the Initial Developer are Copyright (C) 2006-2009
! the Initial Developer. All Rights Reserved.
!
! Contributor(s):
!
! Alternatively, the contents of this file may be used under the terms of
! either the GNU General Public License Version 2 or later (the "GPL"), or
! the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
! in which case the provisions of the GPL or the LGPL are applicable instead
! of those above. If you wish to allow use of your version of this file only
! under the terms of either the GPL or the LGPL, and not to allow others to
! use your version of this file under the terms of the MPL, indicate your
! decision by deleting the provisions above and replace them with the notice
! and other provisions required by the GPL or the LGPL. If you do not delete
! the provisions above, a recipient may use your version of this file under
! the terms of any one of the MPL, the GPL or the LGPL.
!
! ***** END LICENSE BLOCK *****
!
! Contains:
! - subroutine readFile(file,natoms,catoms,tatoms,hybrid)
!
module hybrid_read
  use general, only: r8
  implicit none

  type opt_type
    logical :: showHyb = .false., doPDHyb = .true., tbDOS = .false., &
               tbBand  = .false., symm    = .false.
    integer :: maxiter = 4
  end type opt_type

  type rot_type ! all angles in rad
     logical  :: identity = .true.
     ! rotate first around z1, than x or y and then z2P
     real(r8) :: z1 = 0.0_r8, y = 0.0_r8, x = 0.0_r8, z2 = 0.0_r8
     ! rotation matrix
     real(r8) :: mat(3,3) = reshape( (/ 1.0_r8, 0.0_r8, 0.0_r8, &
                                        0.0_r8, 1.0_r8, 0.0_r8, &
                                        0.0_r8, 0.0_r8, 1.0_r8 /), (/ 3,3 /) )
  end type rot_type

  type band_type
     integer                                     :: n
     character(len=20), dimension(:),pointer :: label
     real(r8),          dimension(:),pointer :: k1, k2, k3
  end type band_type

contains

  subroutine readFile(file,natoms,catoms,tatoms,hybrid,options,rotmat,&
                      shifted,scaled,lat,nkpts,Emin,Emax,numE,band)
    use general, only: lowercase, degree, M_PI
    use atomdata, only: getAtomicNo, ATOM_TYPE,getatomdata
    use slaterKoster, only: hyb_type
    use hybrid_symm, only: rotEuler, rotEulerY
    implicit none
    character(*),intent(in) :: file
    integer,     intent(out):: natoms
! Fortran 2003:
!    real(r8),        allocatable,dimension(:,:),intent(out) :: catoms
!    type(atom_type), allocatable,dimension(:),  intent(out) :: tatoms
!    type(hyb_type),  allocatable,dimension(:,:),intent(out) :: hybrid

    real(r8),        pointer,dimension(:,:) :: catoms
    type(atom_type), pointer,dimension(:)   :: tatoms
    type(hyb_type),  pointer,dimension(:,:) :: hybrid
    type(opt_type),  intent(out)            :: options
    type(rot_type),  intent(out)            :: rotmat
    type(band_type), intent(out)            :: band
    real(r8),        intent(out)            :: shifted(3)
    real(r8),        intent(out)            :: scaled(3),lat(3)
    real(r8),        intent(out)            :: Emin, Emax
    integer,         intent(out)            :: nkpts(3),numE

    integer  :: atom, atom1, atom2, i, nk1, nk2, nk3, maxiter
    real(r8) :: sss, sps, pps, ppp, &
                sds, pds, pdp, dds, ddp, ddd, &
                fsss, fsps, fpps, fppp, fsds, fpds, fpdp, &
                fdds, fddp, fddd
    namelist /hyb/ atom1, atom2, sss, sps, pps, ppp, &
                   sds, pds, pdp, dds, ddp, ddd, &
                   fsss, fsps, fpps, fppp, fsds, fpds, fpdp, &
                   fdds, fddp, fddd
    logical :: showHyb, doPDHyb, tbDOS, tbBand, symm
    namelist /opt/ showHyb, doPDHyb, tbDOS, tbBand, symm, maxiter
    real(r8) :: z1,z2,x,y,z
    logical :: degrees
    namelist /rot/ z1,z2,x,y,degrees
    namelist /shift/ x,y,z
    real(r8) :: a1,a2,a3
    namelist /scale/ a1,a2,a3
    namelist /tb/ a1,a2,a3,nk1,nk2,nk3,Emin,Emax,numE
    real(r8) :: Es, Ep, Ed
    namelist /tbsite/ atom,Es,Ep,Ed
    namelist /autorot/ atom, atom2
    real(r8) :: tmpVec(3)
    character(200)    :: str
    logical           :: l_exists
    integer,parameter :: iounit = 18
    integer           :: iostat

    natoms = 0

    ! initalize options:
    shifted = 0.0_r8
    scaled  = 1.0_r8
    lat = 0.0_r8
    nkpts = 10
    Emin = -10.0_r8
    Emax =  10.0_r8
    numE = 100
    band%n = 0
    degrees = .true.

    if(associated(catoms)) deallocate(catoms)
    if(associated(tatoms)) deallocate(tatoms)
    if(associated(hybrid)) deallocate(hybrid)

    inquire(file=file,exist=l_exists)
    if(l_exists .eqv. .false.) then
        write(*,*) 'ERROR: readFile - cannot open file "',trim(file),'": Does not exist'
        stop
    end if
    open(iounit, file=file, form='formatted', status='old',iostat=iostat)
    if(iostat > 0) then
        write(*,*) 'ERROR: readFile - cannot open file "',trim(file),'"'
        stop
    end if
    do
        ! initialize hybridization
        atom1 = 0; atom2 = 0
        sss = huge(sss)
        sps = sss; pps = sss; ppp = sss; sds = sss
        pds = sss; pdp = sss; dds = sss; ddp = sss; ddd = sss
        fsss = sss; fsps = sss; fpps = sss; fppp = sss; fsds = sss
        fpds = sss; fpdp = sss; fdds = sss; fddp = sss; fddd = sss

        read(iounit,'(a)', iostat=iostat) str
        if(iostat /= 0) then
           exit
        end if
        str = adjustl(str)
        ! skip empty/comment lines
        if(str == '' .or. str(1:1) == '!' .or. str(1:1) == '#' .or. str(1:2) == '//') cycle
        ! is namelist? If yes, read in hybridization
        if(lowercase(str(1:4)) == '&opt') then
           showHyb = options%showHyb
           doPDHyb = options%doPDHyb
           tbDOS   = options%tbDOS
           tbBand  = options%tbBand
           symm    = options%symm
           maxiter = options%maxiter
           backspace(iounit)
           read(iounit,nml=opt)
           options%showHyb = showHyb
           options%doPDHyb = doPDHyb
           options%tbDOS   = tbDOS
           options%tbBand  = tbBand
           options%symm    = symm
           options%maxiter = maxiter
           cycle
        elseif(lowercase(str(1:6)) == '&scale') then
           backspace(iounit)
           a1 = scaled(1); a2 = scaled(2); a3 = scaled(3)
           read(iounit,nml=scale)
           scaled(1) = a1; scaled(2) = a2; scaled(3) = a3
        elseif(lowercase(str(1:7)) == '&tbsite') then
           backspace(iounit)
           atom = 0
           Es = 0.0_r8; Ep = 0.0_r8; Ed = 0.0_r8
           read(iounit,nml=tbsite)
           if(atom <= 0 .or. atom > natoms) then
              write(*,*) 'ERROR: on-site TB parameter: Atom number invalid or non-given'
              stop
           end if
           tatoms(atom)%Es = Es
           tatoms(atom)%Ep = Ep
           tatoms(atom)%Ed = Ed
           cycle
        elseif(lowercase(str(1:3)) == '&tb') then
           backspace(iounit)
           a1  = lat(1);   a2  = lat(2);   a3  = lat(3)
           nk1 = nkpts(1); nk2 = nkpts(2); nk3 = nkpts(3)
           read(iounit,nml=tb)
           lat(1)   = a1;  lat(2)   = a2;  lat(3)   = a3
           nkpts(1) = nk1; nkpts(2) = nk2; nkpts(3) = nk3
           cycle
        elseif(lowercase(str(1:6)) == '&shift') then
          backspace(iounit)
          x = shifted(1); y = shifted(2); z = shifted(3)
          read(iounit,nml=shift)
          shifted(1) = x; shifted(2) = y; shifted(3) = z
          cycle
        elseif(lowercase(str(1:8)) == '&autorot') then
          backspace(iounit)
          atom = -1
          atom2= -1
          read(iounit,nml=autorot)
          if((atom < 1) .or. (atom > natoms)) then
            write(*,*) 'ERROR: Wrong atom number for &autorot: ',atom
            stop 1
          end if
          if(atom2 /= -1 .and. ((atom2 < 1) .or. (atom2 > natoms))) then
            write(*,*) 'ERROR: Wrong atom2 number for &autorot: ',atom2
            stop 1
          elseif(atom2 == atom) then
            write(*,*) 'ERROR: Wrong atom and atom2 numbers the same for &autorot: ',atom
            stop 1
          end if
          rotmat%z2 = 0.0_r8
          x = catoms(1,atom)+shifted(1)
          y = catoms(2,atom)+shifted(2)
          if(x == 0.0_r8 .and. y /= 0.0_r8) then
            rotmat%z1 = M_PI/2.0_r8
          elseif(y /= 0.0_r8) then
            rotmat%z1 = atan(y/x)
          else
            rotmat%z1 = 0.0_r8
          end if
          x = x*cos(rotmat%z1)+y*sin(rotmat%z1)
          z = catoms(3,atom)+shifted(3)
          rotmat%identity = .false.
          if(z == 0.0_r8 .and. x /= 0.0_r8) then
            rotmat%y =  M_PI/2.0_r8
          elseif(x /= 0.0_r8) then
            rotmat%y = atan(x/z)
          else
            rotmat%y = 0.0_r8
          end if
          rotmat%mat = rotEulerY(rotmat%z1, rotmat%y, 0.0_r8)
          if(atom2 /= -1) then
            tmpVec = matmul(rotmat%mat,catoms(:,atom2)+shifted(:))
            if(tmpVec(1) == 0.0_r8 .and. tmpVec(2) /= 0.0_r8) then
!Check:
!              if(tmpVec(2) < 0.0_r8) then
                 rotmat%z2  = M_PI/2.0_r8
!              else
!                 rotmat%z2  = -M_PI/2.0_r8
!              end if
              rotmat%mat = rotEulerY(rotmat%z1, rotmat%y, rotmat%z2)
            elseif(tmpVec(2) /= 0.0_r8) then
              rotmat%z2 = atan(tmpVec(2)/tmpVec(1))
              rotmat%mat = rotEulerY(rotmat%z1, rotmat%y, rotmat%z2)
            end if
          end if
          cycle
        elseif(lowercase(str(1:4)) == '&rot') then
          backspace(iounit)
          if(degrees) then
            z1 = rotmat%z1/degree; x = rotmat%x/degree; y = rotmat%y/degree; z2 = rotmat%z2/degree
          else
            z1 = rotmat%z1; x = rotmat%x; y = rotmat%y; z2 = rotmat%z2
          end if
          read(iounit,nml=rot)
          if(degrees) then
             rotmat%z1 = z1*degree; rotmat%x = x*degree; rotmat%y = y*degree; rotmat%z2 = z2*degree
          else
             rotmat%z1 = z1; rotmat%x = x; rotmat%y = y; rotmat%z2 = z2
          end if
          if(z1 /= 0.0_r8 .or. y /= 0.0d0 .or. x /= 0.0_r8 .or. z2 /= 0.0_r8) then
            if ( y /= 0.0_r8 .and. x /= 0.0_r8) then
               write(*,*) 'ERROR: Can only use (z1,x,z2) OR (z1,y,z2) NOT x AND y'
               stop 1
            elseif(y == 0.0_r8) then
              rotmat%identity = .false.
              rotmat%mat = rotEuler(rotmat%z1, rotmat%x, rotmat%z2)
            else 
              rotmat%identity = .false.
              rotmat%mat = rotEulerY(rotmat%z1, rotmat%y, rotmat%z2)
            end if
          end if
          x = 0.0_r8 ! is also used for &shift
          cycle
        elseif(lowercase(str(1:4)) == '&hyb') then
           if(.not.associated(hybrid)) then
             write(*,*) 'ERROR: readFile - atoms need to be specified before'
             write(*,*) '  hybridization: ',trim(str)
             stop
           end if
           !str = trim(str)//'/'
           ! This is F2003:
           !read(str,nml=hyb)
           backspace(iounit)
           read(iounit,nml=hyb)
           if(atom1 == 0 .or. atom2 == 0) then
             write(*,*) 'ERROR: readFile - hybridization - need to specify'
             write(*,*) '  both "atom1=" and "atom2=":',trim(str)
             stop
           end if
           if(atom1 == atom2) then
             write(*,*) 'ERROR: readFile - hybridization invalid between same atom'
             write(*,*) '  atom1=atom2=',atom1
             stop
           end if
           hybrid(atom1,atom2)%Vsss  = sss;   hybrid(atom1,atom2)%Vsps = sps
           hybrid(atom1,atom2)%Vpps  = pps;   hybrid(atom1,atom2)%Vppp = ppp
           hybrid(atom1,atom2)%Vsds  = sds;   hybrid(atom1,atom2)%Vpds = pds
           hybrid(atom1,atom2)%Vpdp  = pdp;   hybrid(atom1,atom2)%Vdds = dds
           hybrid(atom1,atom2)%Vddp  = ddp;   hybrid(atom1,atom2)%Vddd = ddd

           hybrid(atom1,atom2)%fVsss  = fsss;   hybrid(atom1,atom2)%fVsps = fsps
           hybrid(atom1,atom2)%fVpps  = fpps;   hybrid(atom1,atom2)%fVppp = fppp
           hybrid(atom1,atom2)%fVsds  = fsds;   hybrid(atom1,atom2)%fVpds = fpds
           hybrid(atom1,atom2)%fVpdp  = fpdp;   hybrid(atom1,atom2)%fVdds = fdds
           hybrid(atom1,atom2)%fVddp  = fddp;   hybrid(atom1,atom2)%fVddd = fddd

           hybrid(atom2,atom1) = hybrid(atom1,atom2)
           cycle
        elseif(lowercase(str(1:5)) == '&band') then
           backspace(iounit)
           read(iounit,*) str, band%n
           allocate(band%label(band%n),band%k1(band%n),band%k2(band%n),band%k3(band%n))
           do i = 1, band%n
              read(iounit,*) band%k1(i), band%k2(i), band%k3(i),band%label(i)
           end do
           cycle
        elseif(str(1:1) == '&') then
           write(*,*) 'ERROR: readFile - unknown option:'
           write(*,*) trim(str)
           stop
        end if
        ! well, if not, it must be the number of atoms
        read(str,*) natoms
        if(natoms < 1) then
          write(*,*) 'ERROR: readFile - invalid number of atoms: ',natoms
          stop
        end if
        if(associated(tatoms)) then
          write(*,*) 'ERROR: already read in atom data!'
          stop
        end if
        allocate(tatoms(1:natoms),catoms(1:3,1:natoms),hybrid(natoms,natoms))
        hybrid%Vsss  = huge(sss); hybrid%Vsps = huge(sps)
        hybrid%Vpps  = huge(pps); hybrid%Vppp = huge(ppp)
        hybrid%Vsds  = huge(sds); hybrid%Vpds = huge(pds)
        hybrid%Vpdp  = huge(pdp); hybrid%Vdds = huge(dds)
        hybrid%Vddp  = huge(ddp); hybrid%Vddd = huge(ddd)

        hybrid%fVsss  = huge(fsss); hybrid%fVsps = huge(fsps)
        hybrid%fVpps  = huge(fpps); hybrid%fVppp = huge(fppp)
        hybrid%fVsds  = huge(fsds); hybrid%fVpds = huge(fpds)
        hybrid%fVpdp  = huge(fpdp); hybrid%fVdds = huge(fdds)
        hybrid%fVddp  = huge(fddp); hybrid%fVddd = huge(fddd)
        do i = 1,natoms
          read(iounit,'(a)', iostat=iostat) str
          if(iostat /= 0) then
            exit
          end if
          str = adjustl(str)
          ! skip empty/comment lines
          if(str == '' .or. str(1:1) == '!' .or. str(1:1) == '#') cycle
          read(str,*,iostat=iostat) catoms(:,i), str
          if(iostat /= 0) then
            write(*,'(a,i0,3a)') 'FAILED to read coordinate/atomtype of ', &
                                 'atom number ', i, ' (File: ',trim(file),')'
            stop 3
          end if
          call getAtomData(getAtomicNo(str),tatoms(i))
          tatoms(i)%Es = 0.0_r8
          tatoms(i)%Ep = 0.0_r8
          tatoms(i)%Ed = 0.0_r8
        end do
    end do
    close(iounit)
    if(natoms == 0) then
       write(*,*) 'ERROR: readFile - no atom found!'
       stop
    end if
    if((options%tbDOS .or. options%tbBand) .and. ((lat(1) <= 0.0_r8) &
       .or. (lat(1) <= 0.0_r8) .or. (lat(3) <= 0.0_r8))) then
       write(*,*) 'ERROR: readFile - tightbinding selected and no lattice (&tb a1,a2,a3)  given'
       stop
    end if
    if(options%tbDOS .and. (Emax-Emin) <= 0.0_r8) then
       write(*,*) 'ERROR: readFile - DOS selected and Emin >= Emax; Emin = ',&
             Emin,', Emax = ',Emax
       stop
    end if 
  end subroutine readFile

end module hybrid_read
