!    This file is part of hybrid.
!    $Id: hybrid_symm.f90,v 1.7 2009-02-10 19:38:22 tburnus Exp $
!
! ***** BEGIN LICENSE BLOCK *****
! Version: MPL 1.1/GPL 2.0/LGPL 2.1
!
! The contents of this file are subject to the Mozilla Public License Version
! 1.1 (the "License"); you may not use this file except in compliance with
! the License. You may obtain a copy of the License at
! http://www.mozilla.org/MPL/
!
! Software distributed under the License is distributed on an "AS IS" basis,
! WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
! for the specific language governing rights and limitations under the
! License.
!
! The Original Code is Hybrid
!
! The Initial Developer of the Original Code is Tobias Burnus <burnus@net-b.de>
! Portions created by the Initial Developer are Copyright (C) 2006-2009
! the Initial Developer. All Rights Reserved.
!
! Contributor(s):
!
! Alternatively, the contents of this file may be used under the terms of
! either the GNU General Public License Version 2 or later (the "GPL"), or
! the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
! in which case the provisions of the GPL or the LGPL are applicable instead
! of those above. If you wish to allow use of your version of this file only
! under the terms of either the GPL or the LGPL, and not to allow others to
! use your version of this file under the terms of the MPL, indicate your
! decision by deleting the provisions above and replace them with the notice
! and other provisions required by the GPL or the LGPL. If you do not delete
! the provisions above, a recipient may use your version of this file under
! the terms of any one of the MPL, the GPL or the LGPL.
!
! ***** END LICENSE BLOCK *****
!
! Contains:
! - subroutine symmetry(natoms,catoms)
!
!TODO: Currently, it assumes that all atoms (except (0,0,0) are identically)
! In addition, this routine is very buggy and incomplete! Esp. for D3d
! as the axis is not correctly found!
module hybrid_symm
  use general, only: r8, M_PI, M_PI2
  implicit none
contains
  subroutine symmetry(natoms,catoms,tatoms)
    use atomdata, only: atom_type
    implicit none
    integer,  intent(in)                      :: natoms
    real(r8), dimension(3,natoms), intent(in) :: catoms
    type(atom_type), dimension(natoms),intent(in) :: tatoms
    integer :: i,j,k, naxes
    real(r8)             :: axes(3,2*natoms)
    integer, allocatable :: Caxes(:)
    real(r8) :: temp(3),temp2(3,3)

    naxes = 0

do i=1,natoms
  print *, 'Atom coord: ', catoms(:,i)
end do
    ! Find axes relative to (0,0,0)
    outer: do i = 1,natoms
      if(all(catoms(:,i) == (/0.0, 0.0, 0.0 /))) cycle
      inner: do j = 1,naxes
        if(sum(abs(axes(:,j)*dist(catoms(:,i))-abs(catoms(:,i)))) < 1e-5) cycle outer
      end do inner
      ! Not found, add axis
      naxes = naxes + 1
      print *,'Axes: ',abs(catoms(:,i))/dist(abs(catoms(:,i)))
      axes(:,naxes) = abs(catoms(:,i))/dist(abs(catoms(:,i)))
    end do outer

    ! Create axis perpendicular to those axes
    i = 0
    outer2: do
      i = i + 1
      if(i >= naxes) exit outer2
      j = 0
      middle: do
         j = j + 1
         if(i == j) cycle
         if(j >= naxes) exit middle
         temp = abs(Xproduct(axes(:,j),axes(:,i)))
         k = 0
         inner2: do
            k = k + 1
            if(k > naxes) exit inner2
            if(sum(abs(temp-axes(:,k))) < 1e-5) cycle middle
         end do inner2
         ! Not found, add axis
         naxes = naxes + 1
         axes(:,naxes) = temp
      end do middle
    end do outer2

    allocate(Caxes(naxes))

    do i = 1, naxes
        Caxes(i) = 1
        temp2 = rotToZaxis(axes(:,i))
        if (checkRotation(natoms, catoms, matmul(rotZ(M_PI/3.0_r8),temp2))) then
          Caxes(i) = 6
        else if (checkRotation(natoms, catoms, matmul(rotZ(M_PI2),temp2))) then
          Caxes(i) = 4
        else if (checkRotation(natoms, catoms, matmul(rotZ(M_PI*2.0_r8/3.0_r8),temp2))) then
          Caxes(i) = 3
        else if (checkRotation(natoms, catoms, matmul(rotZ(M_PI),temp2))) then
          Caxes(i) = 2
        end if
    end do

    print *, 'Found ',naxes,' Axes'
    do i = 1, naxes
      print *, axes(:,i), 'C_',Caxes(i)
      print *, 'Inversion symmetry: ', checkmirror(natoms, catoms, rotToZaxis(axes(:,i)),(/-1.0_r8,-1.0_r8,-1.0_r8/))
      print *, 'Mirror symmetry sigma_h: ', checkmirror(natoms, catoms, rotToZaxis(axes(:,i)),(/1.0_r8,1.0_r8,-1.0_r8/))
      print *, 'Mirror symmetry sigma_v: ', checkmirror(natoms, catoms, rotToZaxis(axes(:,i)),(/-1.0_r8,1.0_r8,1.0_r8/))
      print *, 'Mirror symmetry sigma_v: ', checkmirror(natoms, catoms, rotToZaxis(axes(:,i)),(/1.0_r8,-1.0_r8,1.0_r8/))
      print *, ''
    end do

    ! print *, 'Has inversion symmetry'
  contains
    function dist(a)
      real(r8),intent(in) :: a(3)
      real :: dist
      dist = sqrt(a(1)**2+a(2)**2+a(3)**2)
    end function dist

    function Xproduct(a,b)
      real(r8),intent(in) :: a(3),b(3)
      real :: Xproduct(3)
      Xproduct(1) = a(2)*b(3)-b(2)*a(3)
      Xproduct(2) = b(1)*a(3)-a(1)*b(3)
      Xproduct(3) = a(1)*b(2)-b(1)*a(2)
    end function Xproduct
  end subroutine symmetry

  ! Rotate around the X axis by phi (in rad)
  ! cf. http://mathworld.wolfram.com/RotationMatrix.html
  ! Note: In Fortran there are row not column vectors
  function rotX(phi) result(mat)
    implicit none
    real(r8), intent(in)     :: phi
    real(r8), dimension(3,3) :: mat
    mat = 0.0_r8
    mat(1,1) = 1.0_r8
    mat(2,2) =  cos(phi)
    mat(2,3) =  sin(phi) ! Column vector: sin(phi)
    mat(3,2) = -sin(phi)
    mat(3,3) =  cos(phi)
  end function rotX

  ! Rotate around the Y axis by phi (in rad)
  ! cf. http://mathworld.wolfram.com/RotationMatrix.html
  ! Note: In Fortran there are row not column vectors
  function rotY(phi) result(mat)
    implicit none
    real(r8), intent(in)     :: phi
    real(r8), dimension(3,3) :: mat
    mat = 0.0_r8
    mat(2,2) = 1.0_r8
    mat(1,1) =  cos(phi)
    mat(1,3) = -sin(phi) ! Column vector: sin(phi)
    mat(3,1) =  sin(phi)
    mat(3,3) =  cos(phi)
  end function rotY

  ! Rotate around the Z axis by phi (in rad)
  ! cf. http://mathworld.wolfram.com/RotationMatrix.html
  function rotZ(phi) result(mat)
    implicit none
    real(r8), intent(in)     :: phi
    real(r8), dimension(3,3) :: mat
    mat = 0.0_r8
    mat(3,3) = 1.0_r8
    mat(1,1) =  cos(phi)
    mat(1,2) =  sin(phi) !column vector: sin(phi)
    mat(2,1) = -sin(phi)
    mat(2,2) =  cos(phi)
  end function rotZ

  ! Rotate around z,x,z2, angles in rad
  function rotEuler(z,x,z2)
    implicit none
    real(r8), intent(in) :: z,x,z2
    real(r8), dimension(3,3) :: rotEuler
    rotEuler  = matmul(rotZ(z2),matmul(rotX(x),rotZ(z)))
  end function rotEuler

  ! Rotate around z,y,z2, angles in rad
  function rotEulerY(z,y,z2)
    implicit none
    real(r8), intent(in) :: z,y,z2
    real(r8), dimension(3,3) :: rotEulerY
    rotEulerY  = matmul(rotZ(z2),matmul(rotY(y),rotZ(z)))
  end function rotEulerY

  function rotToZaxis(r) ! Assume 0 <= r_i <= 1
    implicit none
    real(r8), intent(in) :: r(3)
    real(r8), dimension(3,3) :: rotToZaxis, tmp
    if ( r(1) < 1e-8) then
      tmp = rotZ(M_PI2)
    else
      tmp = rotZ(atan(r(2)/r(1)))
    end if
    if ( r(3) < 1e-8) then
      rotToZaxis = matmul(transpose(rotY(-M_PI2)),tmp)
    else
      rotToZaxis = matmul(transpose(rotY(-atan(r(1)/r(3)))),tmp)
    end if
  end function rotToZaxis


  function checkRotation(natoms, catoms,rotmat)
    implicit none
    logical :: checkRotation
    integer,  intent(in)                          :: natoms
    real(r8), dimension(3,natoms), intent(in)     :: catoms
    real(r8), dimension(3,3),      intent(in)     :: rotmat
    real(r8), dimension(3,natoms) :: rotAtoms
    integer :: i,j
    ! Rotate
    do i = 1, natoms
      rotAtoms(:,i) = matmul(transpose(rotmat), catoms(:,i))
    end do 
    ! Check
    outer: do i = 1, natoms
      inner: do j = 1, natoms
        if((abs(rotAtoms(1,i)-catoms(1,j)) + abs(rotAtoms(2,i)-catoms(2,j)) &
            +abs(rotAtoms(3,i)-catoms(3,j))) < 10e-5 )&
          exit inner
      end do inner
      if( j > natoms ) then
        checkRotation = .false.
        return
      end if
    end do outer
    checkRotation = .true.
  end function checkRotation

  function checkMirror(natoms, catoms, mult,mirror)
     logical :: checkMirror
     integer, intent(in) :: natoms
     real(r8), intent(in) :: catoms(3,natoms), mult(3,3), mirror(3)
     real(r8) :: tmp(3)
     integer :: i, j
     outer: do i = 1, natoms
       tmp = matmul(mult,catoms(:,i))*mirror
       do j = 1, natoms
         if(all(abs(tmp - matmul(mult,catoms(:,j))) < 1e-5)) cycle outer
       end do
       checkMirror = .false.
       return
     end do outer
     checkMirror = .true.
  end function checkMirror

  ! Faculty
  function fac(x)
    integer(8), intent(in) :: x
    integer(8) :: fac,i
    fac = 1
    do i = 1, x
      fac = fac * i
    end do
  end function fac

  ! Spherical harmonics and symmetry
  function YSymm(l,m, theta, phi)
    implicit none
    logical :: YSymm
    integer,  intent(in) :: l, m
    real(r8), intent(in) :: theta, phi
    integer :: i
    real(r8), dimension(2,4), parameter :: trialAng = &
        reshape( (/ 0.0, 0.0, &
                    1.0, 0.0, &
                    0.0, 1.0, &
                    1.0, 1.0  &
               /), (/ 2, 4/))
    YSymm = .true.
    do i = 1, ubound(trialAng,2)
      if(abs(SphericalHarmonicY(l,m,trialAng(1,i),       trialAng(2,i)) &
            -SphericalHarmonicY(l,m,trialAng(1,i)+theta, trialAng(2,i)+phi)) &
           > 1.0e-5) then
!        print *,'y(l=',l,',m=',m,',theta,phi)=',SphericalHarmonicY(l,m,theta,phi)
!        print *,'y(l=',l,',m=',m,',theta+,phi+)=',SphericalHarmonicY(l,m,trialAng(1,i)+theta, trialAng(2,i)+phi)
        print *, trialAng(1,i), trialAng(2,i), trialAng(1,i)+theta, trialAng(2,i)+phi
        print *,'FALSE: Diff:',l,m,SphericalHarmonicY(l,m,trialAng(1,i),trialAng(2,i)) &
                                  -SphericalHarmonicY(l,m,trialAng(1,i)+theta, trialAng(2,i)+phi)
        YSymm = .false.
        return
      else
!        print *,'TRUE: Diff:',l,m,SphericalHarmonicY(l,m,trialAng(1,i),trialAng(2,i))-SphericalHarmonicY(l,m,trialAng(1,i)+theta, trialAng(2,i)+phi)
      end if
    end do
  end function

  ! Spherical Harmonics
  function SphericalHarmonicY(l,m, theta, phi) result(Y)
   integer, intent(in)  :: l, m
   real(r8), intent(in) :: theta, phi
   complex(r8)             :: Y

   if( l < 0 .or. abs(m) > l) then
     print *, 'ERROR: Y not available for l =',l,'  m = ',m
   else if (l == 0 .and. m == 0) then
     Y = 0.5_r8/sqrt(M_PI)
   else if (l == 1 .and. m == 0) then
     Y = 0.5_r8*sqrt(3.0_r8/M_PI)*cos(theta)
   else if (l == 1) then
     Y = -m*0.5_r8*sqrt(3.0_r8/(2.0_r8*M_PI))*sin(theta)
   else if (l == 2 .and. m == 0) then
     Y = 0.25_r8*sqrt(5.0_r8/M_PI)*(3.0_r8*cos(theta)**2-1.0_r8)
   else if (l == 2 .and. (m == -1 .or. m == 1)) then
     Y = -m*0.5_r8*sqrt(15.0_r8/(2.0_r8*M_PI))*(cos(theta)*sin(theta))
   else if (l == 2) then
     Y = 0.25_r8*sqrt(15.0_r8/(2.0_r8*M_PI))*sin(theta)**2
   else if (l == 4 .and. m == 0) then
     Y = 3.0_r8/(16.0_r8*sqrt(M_PI))*(35.0_r8*cos(theta)**4-30.0_r8*cos(theta)**2+3.0_r8)
   else
     write(0,*) 'ERROR: Y not available for l =',l,'  m = ',m
     Y = 0.0_r8
!    Y = sqrt((2.0_r8*l+1.0_r8)/(4.0_r8*M_PI)*fac(l-m)/real(fac(1+m),r8)) &
!        * P(m,l,cos(theta))
   end if
   if(m /= 0) Y = Y * exp(cmplx(0.0, m*phi))
  end function SphericalHarmonicY
end module hybrid_symm
