!    This file is part of hybrid.
!    $Id: tightbinding.f90,v 1.9 2009-02-10 19:38:22 tburnus Exp $
!
! ***** BEGIN LICENSE BLOCK *****
! Version: MPL 1.1/GPL 2.0/LGPL 2.1
!
! The contents of this file are subject to the Mozilla Public License Version
! 1.1 (the "License"); you may not use this file except in compliance with
! the License. You may obtain a copy of the License at
! http://www.mozilla.org/MPL/
!
! Software distributed under the License is distributed on an "AS IS" basis,
! WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
! for the specific language governing rights and limitations under the
! License.
!
! The Original Code is Hybrid
!
! The Initial Developer of the Original Code is Tobias Burnus <burnus@net-b.de>
! Portions created by the Initial Developer are Copyright (C) 2006-2009
! the Initial Developer. All Rights Reserved.
!
! Contributor(s):
!
! Alternatively, the contents of this file may be used under the terms of
! either the GNU General Public License Version 2 or later (the "GPL"), or
! the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
! in which case the provisions of the GPL or the LGPL are applicable instead
! of those above. If you wish to allow use of your version of this file only
! under the terms of either the GPL or the LGPL, and not to allow others to
! use your version of this file under the terms of the MPL, indicate your
! decision by deleting the provisions above and replace them with the notice
! and other provisions required by the GPL or the LGPL. If you do not delete
! the provisions above, a recipient may use your version of this file under
! the terms of any one of the MPL, the GPL or the LGPL.
!
! ***** END LICENSE BLOCK *****

! Contains:
module tb
  implicit none

contains
  subroutine runTightBinding(natoms, catoms, tatoms, Vhyb, lattice,nkpts,&
      dos,band,filebase,Emin,Emax,numE,bands)
    use general, only: r8
    use atomdata, only: atom_type
    use slaterKoster, only: hyb_type
    use hybrid_read, only: band_type
    integer, intent(in)          :: natoms
    type(atom_type),  intent(in) :: tatoms(natoms)   ! Atom data
    real(r8),         intent(in) :: catoms(3,natoms) ! Atom coordinate
    type(hyb_type),   intent(in) :: Vhyb(natoms,natoms)   ! user-defined hybridization
    type(band_type),  intent(in) :: bands
    real(r8),         intent(in) :: lattice(3),Emin,Emax
    integer,          intent(in) :: nkpts(3),numE
    logical,          intent(in) :: dos,band
    character(len=*), intent(in) :: filebase

    integer, parameter       :: ioDOS = 96, ioDOSp = 97, ioBand = 98,ioBandP = 99
    real(r8), allocatable    :: E(:)
    integer                  :: i, Hsize, k1,k2,k3,j
    real(r8)                 :: bandEmin, bandEmax
    integer                  :: Hoffset(natoms), DosE(numE)

    if(.not. dos .and. .not. band) return
    if(dos) then
       call deleteFile(trim(filebase)//'.dos')
       open(unit=ioDOS,file=trim(filebase)//'.dos',status='new', &
            form='formatted',position='rewind')
       call deleteFile(trim(filebase)//'-dos.gpl')
       open(unit=ioDOSp,file=trim(filebase)//'-dos.gpl',status='new', &
            form='formatted',position='rewind')
    end if
    if(band) then
       call deleteFile(trim(filebase)//'.band')
       open(unit=ioBand,file=trim(filebase)//'.band',status='new', &
            form='formatted',position='rewind')
       call deleteFile(trim(filebase)//'-band.gpl')
       open(unit=ioBandP,file=trim(filebase)//'-band.gpl',status='new', &
            form='formatted',position='rewind')
    end if
    Hsize   = 0 
    ! count number of orbitals
    do i = 1, natoms
      Hoffset(i) = Hsize
      Hsize = Hsize + (tatoms(i)%vsubshell+1)**2 ! = all electrons in the outer shell
    end do
    allocate(E(Hsize))

    DosE = 0
    if(dos) then
      E = huge(E)
      do k1 = 0,nkpts(1)
        do k2 = 0,nkpts(2)
          do k3 = 0,nkpts(3)
            call runKpt(Hsize,Hoffset,real(k1,r8)/nkpts(1),&
                        real(k2,r8)/nkpts(2),real(k3,r8)/nkpts(3),E(:))
            do i = 1,Hsize
              if(E(i) < Emax .and. E(i) > Emin) then
                j = (E(i)-Emin)*numE/(Emax-Emin)
                if(j < 1 .or. j > numE) then
                   write(*,*) 'Bounds wrong: numE=',numE,' j=',j,'E(i)=',E(i),' Emin=',Emin,' Emax=',Emax
                end if
                DosE(j) = DosE(j) + 1
              end if
            end do
          end do
        end do
      end do
      do i = 1,numE
        write(ioDOS,*) (Emax-Emin)*real(i,r8)/real(numE,r8) + Emin, DosE(i)
      end do
    end if !dos

    if(band) then
      do i = 1, bands%n
        call runKpt(Hsize,Hoffset,bands%k1(i),bands%k2(i),bands%k3(i),E(:))
        do j = 1, size(E)
           write(ioBand,'(i0,3000f19.10)') i, E(j)
        end do
      end do
    end if

    close(ioDOS); close(ioDOSp); close(ioBand); close(ioBandP)
    contains

    subroutine deleteFile(filename)
      character(len=*):: filename
      logical         :: l_exists
      inquire(file=filename,exist=l_exists)
      if(l_exists) then
        open(50,file=filename)
        close(50,status='delete')
      end if
    end subroutine deleteFile

    subroutine runKpt(Hsize,Hoffset,k1,k2,k3,E)
      integer,  intent(in) :: Hsize,Hoffset(natoms)
      real(r8), intent(in) :: k1,k2,k3
      real(r8)             :: E(Hsize)

      integer              :: i,j, a1, a2, o1, o2, pt1,pt2, dt1,dt2
      real(r8)             :: H(Hsize,Hsize)

      H = 0.0d0
      ! Fill diagonal
      do i = 1,natoms
        do j = 1,(tatoms(i)%vsubshell+1)**2
          if(j == 1) &
            H(Hoffset(i)+j,Hoffset(i)+j) = H(Hoffset(i)+j,Hoffset(i)+j) + tatoms(i)%Es
          if(j >= 2 .or. j <= 4) &
            H(Hoffset(i)+j,Hoffset(i)+j) = H(Hoffset(i)+j,Hoffset(i)+j) + tatoms(i)%Ep
          if(j >= 5 .or. j <= 10) &
            H(Hoffset(i)+j,Hoffset(i)+j) = H(Hoffset(i)+j,Hoffset(i)+j) + tatoms(i)%Ed
        end do
      end do

      ! Fill overlap matrices
      ! This includes also the overlaps between "r" and "r+a1" or "r-a1".

      do a1 = 1,natoms
        do o1 = 1,tatoms(a1)%vsubshell
           do a2 = i, natoms
             do o2 = 0,tatoms(a2)%vsubshell
               if(o1 == 1) then ! s
                 if    (o2 == 0) then ! ss
                 elseif(o2 == 1) then ! sp
                 elseif(o2 == 2) then ! sd
                 else
                   stop 'ERROR: invalid orbital type'
                 end if
               elseif(j <= 4) then ! p
               end if
             end do
           end do ! second atom
        end do ! orbitals of first atom
      end do ! First atom

!      do i = 1,Hsize
!         print '(100f6.3)',H(i,:)
!      end do

      call findeigenvalues_lapack(Hsize, H, E)
    end subroutine runKpt
  end subroutine runTightBinding


! ------------------------------------------------------------
! Eigen solvers
! ------------------------------------------------------------
! Calculate the eigenvalues and eigenfunctions
! for a given potential
!   m    - mesh
!   pot  - potential
!   nev  - number of eigenvalues to be computed
!   ncv  - number of generated Lanczos vectors < m%points (ARPACK only)


subroutine findeigenvalues_lapack(n, H, E)
    use lapack_int, only:dgeev
    use general, only:  r8
    implicit none
    integer, intent(in)   :: n
    real(r8), intent(in)  :: H(n,n)
    real(r8), intent(out) :: E(n)

    integer :: lda, ldvl, ldvr, lwork, info
    integer :: l
    real(r8), allocatable, dimension(:,:) :: vl, vr
    real(r8), allocatable, dimension(:)   :: ev_i, work

    lda   = n
    ldvl  = 1        ! >= 1 since jobvl = N
    ldvr  = n ! >= n since jobrl = V
    lwork = 24*n ! >= 4*n

    allocate(ev_i(n))
    allocate(vl(ldvl, n), vr(ldvr, n), work(lwork))

! FIXME: Use symmetric version of LAPACK routine
    call dgeev('N', & ! (N|V) do not calculate u(j)**H * A = lambda(j)*u(j)**H
               'V', & ! (N|V) calculate eigenvectors A*v(j) = lambda(j)*v(j)
               n, H, lda, E, ev_i, &
               vl, ldvl, vr, ldvr, work, lwork, info)
end subroutine

end module tb
